# pylint: disable=W,C,R

import cv2
import matplotlib

matplotlib.use("TkAgg")

from PIL import Image


def parse_yolo_format(label_path, dh, dw):
    fl = open(label_path, "r")
    data = fl.readlines()
    fl.close()

    for label in data:
        dt = label
        label, x, y, w, h = map(float, dt.split(" "))
        yield get_bounding_box(label, x, y, w, h, dh, dw)


def get_bounding_box(label, x, y, w, h, dh, dw):
    # Taken from https://github.com/pjreddie/darknet/blob/810d7f797bdb2f021dbe65d2524c2ff6b8ab5c8b/src/image.c#L283-L291
    # via https://stackoverflow.com/questions/44544471/how-to-get-the-coordinates-of-the-bounding-box-in-yolo-object-detection#comment102178409_44592380
    l = int((x - w / 2) * dw)
    r = int((x + w / 2) * dw)
    t = int((y - h / 2) * dh)
    b = int((y + h / 2) * dh)

    if l < 0:
        l = 0
    if r > dw - 1:
        r = dw - 1
    if t < 0:
        t = 0
    if b > dh - 1:
        b = dh - 1
    return label, l, t, r, b


def get_image(image_path):
    img = cv2.imread(image_path)
    dh, dw, _ = img.shape
    print((image_path, dh, dw))
    return img, dh, dw


def get_plot(image_path, label_path=None):
    if label_path is None:
        label_path = image_path.replace("images", "labels").replace(".png", ".txt")
    img, dh, dw = get_image(image_path)
    bounding_box = parse_yolo_format(label_path, dh, dw)
    return plot_bounding_box(image_path, bounding_box)


def plot_bounding_box(image_path, bounding_box, unnormalize=False):
    img, dh, dw = get_image(image_path)
    if unnormalize:
        bounding_box = list(
            map(lambda point: get_bounding_box(point[0], point[1], point[2], point[3], point[4], dh, dw), bounding_box)
        )
    for _, l, t, r, b in bounding_box:
        cv2.rectangle(img, (int(l), int(t)), (int(r), int(b)), (0, 0, 255), 5)
    return img


def plot(image_path, label_path, prefix):
    img = get_plot(image_path, label_path)
    Image.fromarray(img).save("{prefix}_yolo_bbox.png".format(prefix=prefix))
