import tempfile
from typing import Any

import graphene
from graphene_file_upload.scalars import Upload
from yolov3.detect_interface import Detect

from ..containers.core_container import CoreContainer
from .bankvoucher import BankVoucher
from .invoice import Invoice


class BankDocument(graphene.Union):
    class Meta:
        types = (Invoice, BankVoucher)


class UploadBankVoucherMutation(graphene.Mutation):
    class Arguments:  # pylint: disable=R0903
        file = Upload(required=True)

    bankDocument = graphene.List(BankVoucher)

    def mutate(self, info: graphene.ResolveInfo, file: Any):  # pylint: disable=W0613
        model = CoreContainer.model()

        file = preprocess_uploaded_file(file)
        bank_document = model.predict_bankvoucher(file)

        return UploadBankVoucherMutation(
            bankDocument=bank_document,
        )


class UploadInvoiceMutation(graphene.Mutation):
    class Arguments:  # pylint: disable=R0903
        file = Upload(required=True)

    bankDocument = graphene.List(Invoice)

    def mutate(self, info: graphene.ResolveInfo, file: Any):  # pylint: disable=W0613
        model = CoreContainer.model()

        file = preprocess_uploaded_file(file)
        bank_document = model.predict_invoice(file)

        return UploadInvoiceMutation(
            bankDocument=bank_document,
        )


def preprocess_uploaded_file(file):
    is_pdf = "pdf" in file.content_type.lower()
    print(is_pdf)
    with tempfile.NamedTemporaryFile() as f:
        f.write(file.read())
        f.flush()
        file = Detect(load_model=False)._get_image_file(f.name, is_pdf=is_pdf)
        return Detect(load_model=False)._open_image(file)
