from dataclasses import dataclass
from datetime import datetime
from typing import Optional, Union

from validate import Validator  # type: ignore

from .date import Date


@dataclass
class DateAndDueDate:
    date: Union[datetime, str]
    due_date: Union[datetime, str]


class GradePeriod(Validator):
    @staticmethod
    def validate(input_str: DateAndDueDate) -> bool:
        date = GradePeriod().get_date_time(input_str.date)
        due_date = GradePeriod().get_date_time(input_str.due_date)

        if date is None or due_date is None:
            return False

        days_between = (due_date - date).days

        if days_between < 14:
            return False
        return True

    def get_date_time(self, input_data: Union[datetime, str]) -> Optional[datetime]:
        if isinstance(input_data, str):
            input_str: str = input_data
            return Date().get_date(input_str)
        return input_data
