import sys
import unittest

from ..containers.core_container import Container
from ..pdfreader.pdf_reader_mock import PdfReaderMock
from .text_number_field_finder import TextNumberFieldFinder


class TextNumberFieldFinderTest(unittest.TestCase):
    def setUp(self) -> None:
        Container.wire(modules=[sys.modules[__name__]])

    def test_with_number(self) -> None:
        pdf_reader_mock = PdfReaderMock()
        for raw, fields in [("1234", "1234"), ("300003\n\x0c", "300003")]:
            file_name = "number"
            pdf_reader_mock.add_file(file_name, "invoice number {fields}".format(fields=raw))
            assert TextNumberFieldFinder(pdf_reader=pdf_reader_mock).find_in_file(file_name) == [
                "{fields}".format(fields=fields)
            ]

    def test_without_number(self) -> None:
        pdf_reader_mock = PdfReaderMock()
        file_name = "number"
        pdf_reader_mock.add_file(
            file_name,
            "no numbers here",
        )
        assert TextNumberFieldFinder(pdf_reader=pdf_reader_mock).find_in_file(file_name) == []
