from .modulus import Modulus
from .validator import Validator


class AccountNumberValidator(Validator):
    def __init__(self) -> None:
        self.modulus_10 = Modulus(10, [5, 4, 3, 2, 7, 6, 5, 4, 3, 2])
        self.modulus_11 = Modulus(11, [5, 4, 3, 2, 7, 6, 5, 4, 3, 2])

    def validate(self, input_str: str) -> bool:
        account_number = input_str.strip()
        has_correct_length = len(account_number) == 11
        has_correct_check_digit = self._has_correct_check_digit(account_number)
        return has_correct_length and has_correct_check_digit

    def _has_correct_check_digit(self, account_number: str) -> bool:
        check_digit = int(account_number[-1])
        account_number_without_check_digit = account_number[:-1]
        calculated_check_digit_11 = self.modulus_11.calculate_check_digit(account_number_without_check_digit)
        return check_digit == calculated_check_digit_11
