import glob
import os

import matplotlib.pyplot as plt
from pdf_invoice_reader_common.plot_yolo import get_plot, plot_bounding_box
from PIL import Image
from yolov3.default import root_image_dir
from yolov3.detect_interface import Detect

data_config, image_path, json_dir = root_image_dir()

assert os.path.isdir(image_path)
assert os.path.isdir(json_dir)


model_name = "invoice_model"
model = Detect(data_config, prefix=model_name)


def convert_document_to_json(document):
    if type(document) == list:
        document = document[0]
    return {
        key: (getattr(document, key).value if getattr(document, key) is not None else None)
        for key, field in document.__dataclass_fields__.items()
    }


def make_label_plot(img_path):
    global model
    output_directory = os.path.join(os.path.dirname(os.path.abspath(__file__)), "debug")
    os.makedirs(output_directory, exist_ok=True)
    image_output = os.path.join(output_directory, os.path.basename(img_path))
    if os.path.isfile(image_output):
        return
    truth_image = Image.fromarray(get_plot(img_path))
    yolo_plot = model.get_raw_yolo_format(img_path, normalized=True)
    model_image = Image.fromarray(plot_bounding_box(img_path, yolo_plot, unnormalize=True))

    f = plt.figure(figsize=(60, 60))
    f.add_subplot(1, 2, 1)
    plt.imshow(truth_image)
    f.add_subplot(1, 2, 2)
    plt.imshow(model_image)
    plt.savefig(image_output, bbox_inches="tight", pad_inches=0.0, dpi=30)
    plt.cla()
    plt.close("all")


files = glob.glob(os.path.join(image_path, "*.png"))
creditor_files = {}
for i in files:
    creditor_id = os.path.basename(i).split("_")[0]
    creditor_files[creditor_id] = i

for img_path in creditor_files.values():
    """
    json_file_name = os.path.join(json_dir, "/".join(os.path.basename(img_path).split("_")[:2]).replace(".png", ".json"))
    if not json_file_name.endswith(".json"):
        json_file_name += ".json"
    json_dict = {}
    with open(json_file_name, "r") as file:
        json_dict = json.loads(file.read())
    print(img_path)
    print(json_dict)
    _, clean_document = parse_document(
        Detect(data_config, prefix=model_name).get_document_from_path(img_path, prefix=model_name, debug=False)
    )
    """
    try:
        make_label_plot(img_path)
    except Exception as e:
        print(e)
