import argparse
import datetime
import json

from pdfinvoicereader.parse_raw_document import parse_document
from pdfinvoicereader.pdfreader.pdf_reader_mupdf import PdfReaderMupdf
from yolov3.detect_interface import Detect


def get_standard_test_files(root_image_dataset_dir):
    img_path = lambda file_name: "{root_image_dataset_dir}{file_name}.png".format(
        root_image_dataset_dir=root_image_dataset_dir, file_name=file_name
    )
    return list(map(img_path, ["13242_1407353", "12691_989908", "12675_1031373"]))


def convert_to_json(documents):
    data = []
    for index, document in enumerate(documents):
        data.append({})
        for field_key in document.__dataclass_fields__:
            fields = getattr(document, field_key)
            if type(fields) != list:
                fields = [fields]
            for i in enumerate(fields):
                data[index][field_key] = getattr(document, field_key)
                data[index][field_key] = data[index][field_key].value if data[index][field_key] is not None else None
                if (type(data[index][field_key])) == datetime.datetime:
                    data[index][field_key] = data[index][field_key].strftime("%Y-%m-%d")
    data = [i for i in data if len(i) > 0]
    return json.dumps(data)


def clean_plot(file, documents):
    bbox_data = []
    for document in documents:
        for field_key in document.__dataclass_fields__:
            fields = getattr(document, field_key)
            if fields is None:
                continue
            if type(fields) != list:
                print(type(fields))
                fields = [fields]
            if len(documents) == 1 and field_key == "entrybox":
                continue
            for field in fields:
                if field.value is None:
                    continue
                bbox_data.append(sum([field.bbox, [field_key]], []))
    plt, fig = Detect(prefix=opt.name).clean_plot_document(file, bbox_data)
    plt.show()


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--random", default=False, action="store_true", help="random image")
    parser.add_argument("--data_config", type=str, default=None, help="location of config")
    parser.add_argument("--file", type=str, default=None, help="location of config")
    parser.add_argument("--folders", type=str, default=None, help="location pdf/png files")
    parser.add_argument("--name", type=str, default="", help="prefix of checkpoint file")
    parser.add_argument(
        "--test_folder", default=False, action="store_true", help="use the test directory in the root path"
    )
    parser.add_argument(
        "--generate-yolo-outside-dataset",
        default=False,
        action="store_true",
        help="will generate yolo labels for files not in the dataset",
    )
    parser.add_argument("--show-plot", default=False, action="store_true", help="show plot")
    parser.add_argument("--no-class-label", default=False, action="store_true", help="show plot")

    opt = parser.parse_args()

    if opt.show_plot:
        if opt.file:
            Detect(opt.data_config, prefix=opt.name, no_mark_class_label=opt.no_class_label).get_plot(opt.file).show()
        else:
            raise Exception("we only support file argument for plot")
    elif opt.generate_yolo_outside_dataset:
        print(opt)
        for detection in Detect(
            opt.data_config, prefix=opt.name, use_test_dir=False
        ).predict_all_files_outside_data_config(outside_directory=opt.folders):
            print(detection)
    elif opt.test_folder:
        for detection in Detect(opt.data_config, prefix=opt.name, use_test_dir=True).predict_all_files():
            print(detection)
    elif opt.folders:
        for detection in Detect(opt.data_config, prefix=opt.name, images=opt.folders).predict_all_files():
            print(detection)
    elif opt.random:
        raw_document, clean_document = parse_document(
            Detect(opt.data_config, prefix=opt.name).predict_random_image(debug=True, plot=True)
        )
        print(clean_document)
    elif opt.file is not None:
        raw_document, clean_document = parse_document(
            Detect(opt.data_config, prefix=opt.name).get_document_from_path(opt.file, debug=True)
        )
        if opt.file.endswith(".pdf"):
            pdf_text_bbox = PdfReaderMupdf().get_text_bbox(opt.file)
            """
            print(merge(
                raw_document,
                pdf_text_bbox,
                model_page_size=1030
            ))
            """
            print(convert_to_json(clean_document))
            print(clean_plot(opt.file, clean_document))
        #        print(clean_plot(opt.file, raw_document))
        else:
            print(convert_to_json(clean_document))
            print(clean_plot(opt.file, clean_document))
        print(clean_document)
    else:
        for img_path in get_standard_test_files(
            "/home/brage/sambashare/pdf-invoice-reader/projects/pdf_invoice_reader/datasets/pdfs-bounding-box/yolov3-kid-invoice-dates-v12/images/"
        ):
            raw_document, clean_document = parse_document(
                Detect(opt.data_config, prefix=opt.name).get_document_from_path(img_path, prefix=opt.name, debug=True)
            )
            print(clean_document)
