import os

import torch
import torchvision.transforms as transforms
from pdf_invoice_reader_common.field_format import format_field
from PIL import Image
from torch.autograd import Variable

from .eval_model import extract_text_from_image
from .models import *
from .utils.datasets import *
from .utils.transforms import *
from .utils.utils import *


def json_flaten(fields_dict, prefix_key_name=""):
    keys = {}
    for key, value in fields_dict.items():
        resolve_key = lambda key, prefix_key_name: key if key not in keys else prefix_key_name + key
        if type(value) in [str, float, int]:
            keys[resolve_key(key, prefix_key_name)] = value
        elif type(value) == dict:
            for child_key, child_value in json_flaten(
                value, prefix_key_name="{prefix_key_name}{key}".format(prefix_key_name=prefix_key_name, key=key)
            ).items():
                keys[resolve_key(child_key, key)] = child_value
        elif value is None:
            keys[resolve_key(key, prefix_key_name)] = value
        else:
            raise Exception("not implemeneted flatting for type {}".format(type(value)))
    return keys


def get_accracy_of_model(model, image_dataset, dataloader, opt, classes, run_accuracy_test=False, epoch=None):
    model.eval()

    torch.cuda.empty_cache()

    Tensor = torch.cuda.FloatTensor if torch.cuda.is_available() else torch.FloatTensor

    labels = []
    imgs = []
    img_detections = []
    sample_metrics = []
    for batch_i, (path, input_imgs, targets) in enumerate(dataloader):
        input_imgs = Variable(input_imgs.type(Tensor))

        # Get detections
        with torch.no_grad():
            detections = model(input_imgs)
            if epoch is not None:
                conf_thres = min(0.3 + (epoch * 0.1), opt.conf_thres)
            else:
                conf_thres = opt.conf_thres
            detections = non_max_suppression(detections, conf_thres, opt.nms_thres)

        imgs.extend(path)
        img_detections.extend(detections)
        labels += targets[:, 1].tolist()

        # Rescale target
        targets[:, 2:] = xywh2xyxy(targets[:, 2:])
        targets[:, 2:] *= opt.img_size

        sample_metrics += get_batch_statistics(detections, targets, iou_threshold=0.5)

    if len(sample_metrics) == 0:
        return None, (None, None, None, None, None)

    true_positives, pred_scores, pred_labels = [np.concatenate(x, 0) for x in list(zip(*sample_metrics))]
    precision, recall, AP, f1, ap_class = ap_per_class(true_positives, pred_scores, pred_labels, labels)

    accuracy = 0
    if run_accuracy_test:
        test_image_files = 25
        for img_i, (path, detections) in enumerate(zip(imgs, img_detections)):
            if img_i > test_image_files:
                break
            img = np.array(Image.open(path))
            Image.fromarray(img).save(os.path.join("output", "ordinal_no_mask_{}.png".format(img_i)))
            if detections is not None:
                # Rescale boxes to original image
                detections = rescale_boxes(detections, opt.img_size, img.shape[:2])
                unique_labels = detections[:, -1].cpu().unique()
                mask = np.zeros((img.shape[:-1]))
                fields_masks = {}
                field_values = {}
                iizy_field_value = json_flaten(image_dataset.get_json_from_file(path))
                for x1, y1, x2, y2, conf, cls_conf, cls_pred in detections:
                    box_w = int(x2 - x1)
                    box_h = int(y2 - y1)
                    x1 = int(x1)
                    y1 = int(y1)

                    field_key = classes[int(cls_pred)]
                    current_field_value = iizy_field_value.get(field_key, None)
                    if current_field_value is None:
                        continue
                    field_values[field_key] = current_field_value

                    if field_key not in fields_masks:
                        fields_masks[field_key] = mask.copy()
                    fields_masks[field_key][y1 : y1 + box_h, x1 : x1 + box_w] = 255

                found_count = 0
                for field_key, field_mask in fields_masks.items():
                    applied_mask = img.copy()
                    field_mask[field_mask > 0] = 1
                    applied_mask[~field_mask.astype(bool), :] = 0

                    teseract_text = extract_text_from_image(transforms.ToPILImage()(applied_mask))
                    max_length_text = 100
                    field_values[field_key]

                    for formated_value in format_field(field_key, field_values[field_key]):
                        found_count += int(formated_value in teseract_text and len(teseract_text) < max_length_text)
                accuracy += found_count / len(field_values)

    model.train()
    return accuracy / len(imgs), (precision, recall, AP, f1, ap_class)
