import functools
import multiprocessing.pool

import pytesseract


def combine_mask(x, y):
    x[:, y[0][1] > 0] = 0
    return x


def extract_text_from_image(img):
    custom_config = r"--oem 3 --psm 6"
    return pytesseract.image_to_string(img, config=custom_config)


def timeout(max_timeout):
    """Timeout decorator, parameter in seconds."""

    def timeout_decorator(item):
        """Wrap the original function."""

        @functools.wraps(item)
        def func_wrapper(*args, **kwargs):
            """Closure for function."""
            pool = multiprocessing.pool.ThreadPool(processes=1)
            async_result = pool.apply_async(item, args, kwargs)
            # raises a TimeoutError if execution exceeds max_timeout
            return async_result.get(max_timeout)

        return func_wrapper

    return timeout_decorator
