import os
import time

import torchvision.transforms as transforms
from torchvision.utils import save_image

from .default import data_config as data_config_file
from .utils.datasets import ListDataset, get_trainable_files
from .utils.parse_config import *
from .utils.transforms import PadSquare

current_dir = os.path.dirname(os.path.abspath(__file__))


def run_augmentation(basename, image=None, data_config_file=None, output_dir=None):
    if image is None:
        data_config = parse_data_config(data_config_file)
        root_path = os.path.join(current_dir, data_config["root_folder"])
        image_path = os.path.join(root_path, "images")
        os.path.join(root_path, "classes.names")

        images = get_trainable_files(image_path, None, None)[:10]
    else:
        images = [image]
    dataset = ListDataset(
        images,
        multiscale=True,
        img_size=1030,
        transform=transforms.Compose([PadSquare(), transforms.ToTensor()]),
        image_only=True,
    )
    for i in range(len(images)):
        basename_with_index = os.path.join(
            output_dir, "{basename}_{i}_{time}".format(basename=basename, i=i, time=time.time())
        )
        img = dataset[i]
        print(img)
        save_image(img, "{basename_with_index}.png".format(basename_with_index=basename_with_index))
        print("stored, {basename_with_index}.png".format(basename_with_index=basename_with_index))


if __name__ == "__main__":
    #    data_config_file = os.path.join(current_dir, "config/custom_bankvoucher.data")
    data_config_file = os.path.join(current_dir, "config/custom.data")
    run_augmentation(
        basename="augmentation_",
        image="/home/brage/Documents/ntnu.png",
        output_dir="generate-augmentation-dataset"
        # data_config_file=data_config_file, output_dir="generate-augmentation-dataset"
    )
